/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "client.h"
#include "util/memory.h"
#include "assert/assert.h"

client_t CLIENT;

void clientInit() {
  memoryZero(&CLIENT, sizeof(client_t));
}

void clientConnect(const clientconnect_t connect) {
  int32_t error;

  assertTrue(
    CLIENT.state == CLIENT_STATE_DISCONNECTED,
    "Client is not in a disconnected state."
  );
  
  // Reset state
  packetQueueInit(&CLIENT.queue);
  CLIENT.error = CLIENT_ERROR_NO_ERROR;
  CLIENT.state = CLIENT_STATE_CONNECTING;
  CLIENT.type = connect.type;
  
  // Init the properties based on the type of client.
  switch(connect.type) {
    case CLIENT_TYPE_NETWORKED:
      networkClientConnect(&CLIENT, connect);
      break;

    case CLIENT_TYPE_SINGLE_PLAYER:
      break;

    default:
      assertUnreachable("Invalid client connection type.");
      break;
  }

  // Spawn the connect thread which will be responsible from here on out.
  error = pthread_create(
    &CLIENT.connectThread,
    NULL,
    (void *(*)(void *))clientThreadConnect,
    &CLIENT
  );
  if(error != 0) {
    CLIENT.state = CLIENT_STATE_DISCONNECTED;
    CLIENT.error = CLIENT_ERROR_CONNECT_THREAD_SPAWN_FAILED;
  }
}

void clientDisconnect() {
  if(CLIENT.state == CLIENT_STATE_DISCONNECTED) return;
  if(CLIENT.state == CLIENT_STATE_DISCONNECTING) return;
}

void clientUpdate() {
  assertIsMainThread("Client update called from non-main thread.");
}

void clientDispose() {
  if(CLIENT.state != CLIENT_STATE_DISCONNECTED) {
    clientDisconnect();
  }
}

void * clientThreadConnect(void *arg) {
  assertNotNull(arg, "Client thread connect argument is null.");
  assertNotMainThread("Client thread connect called from main thread.");

  client_t *client = (client_t *)arg;
  assertTrue(client == &CLIENT, "Invalid client thread arg.");
  
  if(client->state) return NULL;
  
  switch(client->type) {
    case CLIENT_TYPE_NETWORKED:
      break;

    case CLIENT_TYPE_SINGLE_PLAYER:
      break;

    default:
      assertUnreachable("Invalid client type.");
      break;
  }

  //
}